SystemOrganization addCategory: #'Idiom-Exercise'!

TestCase subclass: #IdiomTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Idiom-Exercise'!

!IdiomTest methodsFor: 'testing' stamp: 'IgalFrid 5/9/2012 20:16'!
test01AddingCustomerShouldNotTakeMoreThan50Milliseconds
	| customerBook|
	
	customerBook := CustomerBook  new.
	
	self verify: [customerBook addCustomerNamed: 'John Lennon'.] ShouldNotTakeMoreThan: 50
! !

!IdiomTest methodsFor: 'testing' stamp: 'IgalFrid 5/9/2012 20:18'!
test02RemovingCustomerShouldNotTakeMoreThan100Milliseconds
	| customerBook johnLennon |
	
	customerBook := CustomerBook  new.
	johnLennon := 'John Lennon'.
	
	customerBook addCustomerNamed: johnLennon.
	
	self verify: [customerBook removeCustomerNamed: johnLennon.] ShouldNotTakeMoreThan: 100
! !

!IdiomTest methodsFor: 'testing' stamp: 'IgalFrid 5/9/2012 20:13'!
test03CanNotAddACustomerWithEmptyName 

	| customerBook|
			
	customerBook := CustomerBook new.

	self try: [customerBook addCustomerNamed: ''] onException: Error do:  [ :anError | 
			self assert: anError messageText = CustomerBook customerCanNotBeEmptyErrorMessage.
			self assert: customerBook isEmpty ]! !

!IdiomTest methodsFor: 'testing' stamp: 'IgalFrid 5/9/2012 20:14'!
test04CanNotAddRepeatedCustomer 
	
	| customerBook johnLennon|
			
	customerBook := CustomerBook new.
	johnLennon := 'John Lennon'.
	customerBook addCustomerNamed: johnLennon.

	self try: [customerBook removeCustomerNamed: 'Paul McCartney'] onException: NotFound do:   [ :anError | 
				self assert: customerBook numberOfCustomers = 1.
				self assert: (customerBook includesCustomerNamed: johnLennon) ]! !

!IdiomTest methodsFor: 'testing' stamp: 'IgalFrid 5/9/2012 20:13'!
try: aBlock onException: anException do: anAlternativeBlock

	[ aBlock value.
	self fail ]
		on: anException 
		do: anAlternativeBlock! !

!IdiomTest methodsFor: 'testing' stamp: 'IgalFrid 5/9/2012 20:15'!
verify: aBlock ShouldNotTakeMoreThan: millisecondsMaxTake
	| millisecondsBeforeRunning millisecondsAfterRunning|
	
	millisecondsBeforeRunning := Time millisecondClockValue.
	aBlock value.
	millisecondsAfterRunning := Time millisecondClockValue.
	
	self assert: (millisecondsAfterRunning-millisecondsBeforeRunning) <  millisecondsMaxTake
	! !

Object subclass: #CustomerBook
	instanceVariableNames: 'customers'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Idiom-Exercise'!

!CustomerBook class methodsFor: 'error messages' stamp: 'HernanWilkinson 7/6/2011 17:57'!
customerAlreadyExistsErrorMessage

	^'Customer already exists'! !

!CustomerBook class methodsFor: 'error messages' stamp: 'HernanWilkinson 7/6/2011 17:53'!
customerCanNotBeEmptyErrorMessage

	^'Customer name cannot be empty'! !

!CustomerBook methodsFor: 'customer management' stamp: 'HernanWilkinson 7/6/2011 17:42'!
addCustomerNamed: aName

	aName isEmpty ifTrue: [ self signalCustomerNameCannotBeEmpty ].
	(customers includes: aName) ifTrue: [ self signalCustomerAlreadyExists ].
	
	customers add: aName ! !

!CustomerBook methodsFor: 'testing' stamp: 'HernanWilkinson 7/6/2011 17:56'!
includesCustomerNamed: aName

	^customers includes: aName ! !

!CustomerBook methodsFor: 'initialization' stamp: 'HernanWilkinson 7/6/2011 17:42'!
initialize

	super initialize.
	customers := OrderedCollection new! !

!CustomerBook methodsFor: 'testing' stamp: 'HernanWilkinson 7/6/2011 17:48'!
isEmpty
	
	^customers isEmpty  ! !

!CustomerBook methodsFor: 'customer management' stamp: 'HernanWilkinson 7/6/2011 17:56'!
numberOfCustomers
	
	^customers size! !

!CustomerBook methodsFor: 'customer management' stamp: 'HernanWilkinson 7/6/2011 17:44'!
removeCustomerNamed: aName
 
	customers remove: aName ! !

!CustomerBook methodsFor: 'customer management' stamp: 'HernanWilkinson 7/6/2011 17:52'!
signalCustomerAlreadyExists 

	self error: self class customerAlreadyExistsErrorMessage! !

!CustomerBook methodsFor: 'customer management' stamp: 'HernanWilkinson 7/6/2011 17:51'!
signalCustomerNameCannotBeEmpty 

	self error: self class customerCanNotBeEmptyErrorMessage ! !
